@@ -402,11 +402,13 @@ process_config_every_time (BusContext      *context,
 {
   DBusString full_address;
   DBusList *link;
-  
+  char *addr;
+
   dbus_bool_t retval;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
+  addr = NULL;
   retval = FALSE;
 
   if (!_dbus_string_init (&full_address))
@@ -427,8 +429,6 @@ process_config_every_time (BusContext      *context,
   link = _dbus_list_get_last_link (&context->servers);
   while (link != NULL)
     {
-      char *addr;
-      
       addr = dbus_server_get_address (link->data);
       if (addr == NULL)
         {
@@ -452,6 +452,7 @@ process_config_every_time (BusContext      *context,
         }
 
       dbus_free (addr);
+      addr = NULL;
 
       link = _dbus_list_get_prev_link (&context->servers, link);
     }
@@ -489,6 +490,10 @@ process_config_every_time (BusContext      *context,
 
  failed:
   _dbus_string_free (&full_address);
+  
+  if (addr)
+    dbus_free (addr);
+
   return retval;
 }
 
@@ -526,7 +531,8 @@ bus_context_new (const DBusString *config_file,
 {
   BusContext *context;
   BusConfigParser *parser;
-  
+  DBusCredentials creds;
+
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
   context = NULL;
@@ -624,7 +630,7 @@ bus_context_new (const DBusString *config_file,
         }
 
       bytes = _dbus_string_get_length (&addr);
-      if (_dbus_write (print_addr_fd, &addr, 0, bytes) != bytes)
+      if (_dbus_write_socket (print_addr_fd, &addr, 0, bytes) != bytes)
         {
           dbus_set_error (error, DBUS_ERROR_FAILED,
                           "Printing message bus address: %s\n",
@@ -634,7 +640,7 @@ bus_context_new (const DBusString *config_file,
         }
 
       if (print_addr_fd > 2)
-        _dbus_close (print_addr_fd, NULL);
+        _dbus_close_socket (print_addr_fd, NULL);
 
       _dbus_string_free (&addr);
     }
@@ -652,7 +658,25 @@ bus_context_new (const DBusString *config_file,
       BUS_SET_OOM (error);
       goto failed;
     }
-  
+
+  /* check user before we fork */
+  if (context->user != NULL)
+    {
+      DBusString u;
+
+      _dbus_string_init_const (&u, context->user);
+
+      if (!_dbus_credentials_from_username (&u, &creds) ||
+          creds.uid < 0 ||
+          creds.gid < 0)
+        {
+          dbus_set_error (error, DBUS_ERROR_FAILED,
+                          "Could not get UID and GID for username \"%s\"",
+                          context->user);
+          goto failed;
+        }
+    }
+
   /* Now become a daemon if appropriate */
   if ((force_fork != FORK_NEVER && context->fork) || force_fork == FORK_ALWAYS)
     {
@@ -707,7 +731,7 @@ bus_context_new (const DBusString *config_file,
         }
 
       bytes = _dbus_string_get_length (&pid);
-      if (_dbus_write (print_pid_fd, &pid, 0, bytes) != bytes)
+      if (_dbus_write_socket (print_pid_fd, &pid, 0, bytes) != bytes)
         {
           dbus_set_error (error, DBUS_ERROR_FAILED,
                           "Printing message bus PID: %s\n",
@@ -717,7 +741,7 @@ bus_context_new (const DBusString *config_file,
         }
 
       if (print_pid_fd > 2)
-        _dbus_close (print_pid_fd, NULL);
+        _dbus_close_socket (print_pid_fd, NULL);
       
       _dbus_string_free (&pid);
     }
@@ -744,21 +768,6 @@ bus_context_new (const DBusString *config_file,
    */
   if (context->user != NULL)
     {
-      DBusCredentials creds;
-      DBusString u;
-
-      _dbus_string_init_const (&u, context->user);
-
-      if (!_dbus_credentials_from_username (&u, &creds) ||
-          creds.uid < 0 ||
-          creds.gid < 0)
-        {
-          dbus_set_error (error, DBUS_ERROR_FAILED,
-                          "Could not get UID and GID for username \"%s\"",
-                          context->user);
-          goto failed;
-        }
-      
       if (!_dbus_change_identity (creds.uid, creds.gid, error))
 	{
 	  _DBUS_ASSERT_ERROR_IS_SET (error);
