@@ -24,7 +24,7 @@
 
 #include "dbus-internals.h"
 #include "dbus-server-debug-pipe.h"
-#include "dbus-transport-unix.h"
+#include "dbus-transport-socket.h"
 #include "dbus-connection-internal.h"
 #include "dbus-hash.h"
 #include "dbus-string.h"
@@ -256,12 +256,12 @@ _dbus_transport_debug_pipe_new (const char     *server_name,
   _dbus_fd_set_close_on_exec (client_fd);
   _dbus_fd_set_close_on_exec (server_fd);
   
-  client_transport = _dbus_transport_new_for_fd (client_fd,
-                                                 NULL, &address);
+  client_transport = _dbus_transport_new_for_socket (client_fd,
+                                                     NULL, &address);
   if (client_transport == NULL)
     {
-      _dbus_close (client_fd, NULL);
-      _dbus_close (server_fd, NULL);
+      _dbus_close_socket (client_fd, NULL);
+      _dbus_close_socket (server_fd, NULL);
       dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
       _dbus_string_free (&address);
       return NULL;
@@ -271,12 +271,12 @@ _dbus_transport_debug_pipe_new (const char     *server_name,
   
   client_fd = -1;
 
-  server_transport = _dbus_transport_new_for_fd (server_fd,
-                                                 &server->guid_hex, NULL);
+  server_transport = _dbus_transport_new_for_socket (server_fd,
+                                                     &server->guid_hex, NULL);
   if (server_transport == NULL)
     {
       _dbus_transport_unref (client_transport);
-      _dbus_close (server_fd, NULL);
+      _dbus_close_socket (server_fd, NULL);
       dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
       return NULL;
     }
@@ -317,11 +317,106 @@ _dbus_transport_debug_pipe_new (const char     *server_name,
   /* If no one grabbed a reference, the connection will die,
    * and the client transport will get an immediate disconnect
    */
+  _dbus_connection_close_if_only_one_ref (connection);
   dbus_connection_unref (connection);
 
   return client_transport;
 }
 
+/**
+ * Tries to interpret the address entry as a debug pipe entry.
+ * 
+ * Sets error if the result is not OK.
+ * 
+ * @param entry an address entry
+ * @param a new DBusServer, or #NULL on failure.
+ * @param error location to store rationale for failure on bad address
+ * @returns the outcome
+ * 
+ */
+DBusServerListenResult
+_dbus_server_listen_debug_pipe (DBusAddressEntry *entry,
+                                DBusServer      **server_p,
+                                DBusError        *error)
+{
+  const char *method;
+
+  *server_p = NULL;
+  
+  method = dbus_address_entry_get_method (entry);
+  
+  if (strcmp (method, "debug-pipe") == 0)
+    {
+      const char *name = dbus_address_entry_get_value (entry, "name");
+      
+      if (name == NULL)
+        {
+          _dbus_set_bad_address(error, "debug-pipe", "name",
+                                NULL);
+          return DBUS_SERVER_LISTEN_BAD_ADDRESS;
+        }
+
+      *server_p = _dbus_server_debug_pipe_new (name, error);
+      
+      if (*server_p)
+        {
+          _DBUS_ASSERT_ERROR_IS_CLEAR(error);
+          return DBUS_SERVER_LISTEN_OK;
+        }
+      else
+        {
+          _DBUS_ASSERT_ERROR_IS_SET(error);
+          return DBUS_SERVER_LISTEN_DID_NOT_CONNECT;
+        }
+    }
+  else
+    {
+      _DBUS_ASSERT_ERROR_IS_CLEAR(error);
+      return DBUS_SERVER_LISTEN_NOT_HANDLED;
+    }
+}
+
+DBusTransportOpenResult
+_dbus_transport_open_debug_pipe (DBusAddressEntry  *entry,
+                                 DBusTransport    **transport_p,
+                                 DBusError         *error)
+{
+  const char *method;
+  
+  method = dbus_address_entry_get_method (entry);
+  _dbus_assert (method != NULL);
+
+  if (strcmp (method, "debug-pipe") == 0)
+    {
+      const char *name = dbus_address_entry_get_value (entry, "name");
+
+      if (name == NULL)
+        {
+          _dbus_set_bad_address (error, "debug-pipe", "name",
+                                 NULL);
+          return DBUS_TRANSPORT_OPEN_BAD_ADDRESS;
+        }
+          
+      *transport_p = _dbus_transport_debug_pipe_new (name, error);
+
+      if (*transport_p == NULL)
+        {
+          _DBUS_ASSERT_ERROR_IS_SET (error);
+          return DBUS_TRANSPORT_OPEN_DID_NOT_CONNECT;
+        }
+      else
+        {
+          _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+          return DBUS_TRANSPORT_OPEN_OK;
+        }      
+    }
+  else
+    {
+      _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+      return DBUS_TRANSPORT_OPEN_NOT_HANDLED;
+    }
+}
+
 
 /** @} */
 
