@@ -1,5 +1,5 @@
 /* -*- mode: C; c-file-style: "gnu" -*- */
-/* dbus-string.h String utility class (internal to D-BUS implementation)
+/* dbus-string.h String utility class (internal to D-Bus implementation)
  * 
  * Copyright (C) 2002, 2003 Red Hat, Inc.
  *
@@ -39,10 +39,10 @@ DBUS_BEGIN_DECLS
  */
 struct DBusString
 {
-  void *dummy1; /**< placeholder */
-  int   dummy2; /**< placeholder */
-  int   dummy3; /**< placeholder */
-  int   dummy4; /**< placeholder */
+  const void *dummy1; /**< placeholder */
+  int   dummy2;       /**< placeholder */
+  int   dummy3;       /**< placeholder */
+  int   dummy4;       /**< placeholder */
   unsigned int dummy5 : 1; /**< placeholder */
   unsigned int dummy6 : 1; /**< placeholder */
   unsigned int dummy7 : 1; /**< placeholder */
@@ -54,9 +54,12 @@ struct DBusString
  * to inline non-exported symbols across files in the library.
  * Note that these break type safety (due to the casts)
  */
+#define _dbus_string_get_data(s) ((char*)(((DBusString*)(s))->dummy1))
 #define _dbus_string_get_length(s) (((DBusString*)(s))->dummy2)
+#define _dbus_string_set_byte(s, i, b) ((((unsigned char*)(((DBusString*)(s))->dummy1))[(i)]) = (unsigned char) (b))
 #define _dbus_string_get_byte(s, i) (((const unsigned char*)(((DBusString*)(s))->dummy1))[(i)])
 #define _dbus_string_get_const_data(s) ((const char*)(((DBusString*)(s))->dummy1))
+#define _dbus_string_get_const_data_len(s,start,len) (((const char*)(((DBusString*)(s))->dummy1)) + (start))
 #endif
 
 dbus_bool_t   _dbus_string_init                  (DBusString        *str);
@@ -69,19 +72,25 @@ dbus_bool_t   _dbus_string_init_preallocated     (DBusString        *str,
                                                   int                allocate_size);
 void          _dbus_string_free                  (DBusString        *str);
 void          _dbus_string_lock                  (DBusString        *str);
+#ifndef _dbus_string_get_data
 char*         _dbus_string_get_data              (DBusString        *str);
+#endif /* _dbus_string_get_data */
 #ifndef _dbus_string_get_const_data
 const char*   _dbus_string_get_const_data        (const DBusString  *str);
 #endif /* _dbus_string_get_const_data */
 char*         _dbus_string_get_data_len          (DBusString        *str,
                                                   int                start,
                                                   int                len);
+#ifndef _dbus_string_get_const_data_len
 const char*   _dbus_string_get_const_data_len    (const DBusString  *str,
                                                   int                start,
                                                   int                len);
+#endif
+#ifndef _dbus_string_set_byte
 void          _dbus_string_set_byte              (DBusString        *str,
                                                   int                i,
                                                   unsigned char      byte);
+#endif
 #ifndef _dbus_string_get_byte
 unsigned char _dbus_string_get_byte              (const DBusString  *str,
                                                   int                start);
@@ -147,6 +156,9 @@ dbus_bool_t   _dbus_string_append_printf         (DBusString        *str,
 dbus_bool_t   _dbus_string_append_printf_valist  (DBusString        *str,
                                                   const char        *format,
                                                   va_list            args);
+dbus_bool_t   _dbus_string_insert_2_aligned      (DBusString        *str,
+                                                  int                insert_at,
+                                                  const unsigned char octets[2]);
 dbus_bool_t   _dbus_string_insert_4_aligned      (DBusString        *str,
                                                   int                insert_at,
                                                   const unsigned char octets[4]);
@@ -199,11 +211,6 @@ dbus_bool_t   _dbus_string_parse_double          (const DBusString  *str,
                                                   int                start,
                                                   double            *value,
                                                   int               *end_return);
-dbus_bool_t   _dbus_string_parse_basic_type      (const DBusString  *str,
-						  char               type,
-                                                  int                start,
-                                                  void              *value,
-                                                  int               *end_return);
 dbus_bool_t   _dbus_string_find                  (const DBusString  *str,
                                                   int                start,
                                                   const char        *substr,
@@ -226,6 +233,9 @@ void          _dbus_string_skip_blank            (const DBusString  *str,
 void          _dbus_string_skip_white            (const DBusString  *str,
                                                   int                start,
                                                   int               *end);
+void          _dbus_string_skip_white_reverse    (const DBusString  *str,
+                                                  int                end,
+                                                  int               *start);
 dbus_bool_t   _dbus_string_equal                 (const DBusString  *a,
                                                   const DBusString  *b);
 dbus_bool_t   _dbus_string_equal_c_str           (const DBusString  *a,
@@ -246,6 +256,9 @@ dbus_bool_t   _dbus_string_pop_line              (DBusString        *source,
                                                   DBusString        *dest);
 void          _dbus_string_delete_first_word     (DBusString        *str);
 void          _dbus_string_delete_leading_blanks (DBusString        *str);
+void          _dbus_string_chop_white            (DBusString        *str); 
+dbus_bool_t   _dbus_string_append_byte_as_hex    (DBusString        *str,
+                                                  int                byte);
 dbus_bool_t   _dbus_string_hex_encode            (const DBusString  *source,
                                                   int                start,
                                                   DBusString        *dest,
@@ -264,27 +277,32 @@ dbus_bool_t   _dbus_string_validate_utf8         (const DBusString  *str,
 dbus_bool_t   _dbus_string_validate_nul          (const DBusString  *str,
                                                   int                start,
                                                   int                len);
-dbus_bool_t   _dbus_string_validate_path         (const DBusString  *str,
-                                                  int                start,
-                                                  int                len);
-dbus_bool_t   _dbus_string_validate_interface    (const DBusString  *str,
-                                                  int                start,
-                                                  int                len);
-dbus_bool_t   _dbus_string_validate_member       (const DBusString  *str,
-                                                  int                start,
-                                                  int                len);
-dbus_bool_t   _dbus_string_validate_error_name   (const DBusString  *str,
-                                                  int                start,
-                                                  int                len);
-dbus_bool_t   _dbus_string_validate_service      (const DBusString  *str,
-                                                  int                start,
-                                                  int                len);
-dbus_bool_t   _dbus_string_validate_signature    (const DBusString  *str,
-                                                  int                start,
-                                                  int                len);
 void          _dbus_string_zero                  (DBusString        *str);
 
 
+/**
+ * We allocate 1 byte for nul termination, plus 7 bytes for possible
+ * align_offset, so we always need 8 bytes on top of the string's
+ * length to be in the allocated block.
+ */
+#define _DBUS_STRING_ALLOCATION_PADDING 8
+
+/**
+ * Defines a static const variable with type #DBusString called "name"
+ * containing the given string literal.
+ *
+ * @param name the name of the variable
+ * @param str the string value
+ */
+#define _DBUS_STRING_DEFINE_STATIC(name, str)                           \
+  static const char _dbus_static_string_##name[] = str;                 \
+  static const DBusString name = { _dbus_static_string_##name,          \
+                                   sizeof(_dbus_static_string_##name),  \
+                                   sizeof(_dbus_static_string_##name) + \
+                                   _DBUS_STRING_ALLOCATION_PADDING,     \
+                                   sizeof(_dbus_static_string_##name),  \
+                                   TRUE, TRUE, FALSE, 0 }
+
 DBUS_END_DECLS
 
 #endif /* DBUS_STRING_H */
