@@ -22,6 +22,7 @@
  *
  */
 #include "dbus-sysdeps.h"
+#include "dbus-sysdeps-unix.h"
 #include "dbus-internals.h"
 #include "dbus-protocol.h"
 #include "dbus-string.h"
@@ -43,6 +44,10 @@
 #include <dirent.h>
 #include <sys/un.h>
 
+#ifdef HAVE_SYS_SYSLIMITS_H
+#include <sys/syslimits.h>
+#endif
+
 #ifndef O_BINARY
 #define O_BINARY 0
 #endif
@@ -152,7 +157,7 @@ _dbus_become_daemon (const DBusString *pidfile,
 	    }
 	  
 	  bytes = _dbus_string_get_length (&pid);
-	  if (_dbus_write (print_pid_fd, &pid, 0, bytes) != bytes)
+	  if (_dbus_write_socket (print_pid_fd, &pid, 0, bytes) != bytes)
 	    {
 	      dbus_set_error (error, DBUS_ERROR_FAILED,
 			      "Printing message bus PID: %s\n",
@@ -502,15 +507,20 @@ static dbus_bool_t
 dirent_buf_size(DIR * dirp, size_t *size)
 {
  long name_max;
-#   if defined(HAVE_FPATHCONF) && defined(HAVE_DIRFD) \
-    && defined(_PC_NAME_MAX)
-     name_max = fpathconf(dirfd(dirp), _PC_NAME_MAX);
+#   if defined(HAVE_FPATHCONF) && defined(_PC_NAME_MAX)
+#      if defined(HAVE_DIRFD)
+          name_max = fpathconf(dirfd(dirp), _PC_NAME_MAX);
+#      else
+          name_max = fpathconf(dirp->dd_fd, _PC_NAME_MAX);
+#      endif /* HAVE_DIRFD */
      if (name_max == -1)
 #           if defined(NAME_MAX)
 	     name_max = NAME_MAX;
 #           else
 	     return FALSE;
 #           endif
+#   elif defined(MAXNAMELEN)
+     name_max = MAXNAMELEN;
 #   else
 #       if defined(NAME_MAX)
 	 name_max = NAME_MAX;
