@@ -23,6 +23,7 @@
 
 #include "dbus-transport-protected.h"
 #include "dbus-transport-unix.h"
+#include "dbus-transport-socket.h"
 #include "dbus-connection-internal.h"
 #include "dbus-watch.h"
 #include "dbus-auth.h"
@@ -200,6 +201,123 @@ _dbus_transport_finalize_base (DBusTransport *transport)
   dbus_free (transport->expected_guid);
 }
 
+
+/**
+ * Verifies if a given D-Bus address is a valid address
+ * by attempting to connect to it. If it is, returns the
+ * opened DBusTransport object. If it isn't, returns #NULL
+ * and sets @p error.
+ *
+ * @param error address where an error can be returned.
+ * @returns a new transport, or #NULL on failure.
+ */
+static DBusTransport*
+check_address (const char *address, DBusError *error)
+{
+  DBusAddressEntry **entries;
+  DBusTransport *transport = NULL;
+  int len, i;
+
+  _dbus_assert (address != NULL);
+  _dbus_assert (*address != '\0');
+
+  if (!dbus_parse_address (address, &entries, &len, error))
+    return FALSE;              /* not a valid address */
+
+  for (i = 0; i < len; i++)
+    {
+      transport = _dbus_transport_open (entries[i], error);
+      if (transport != NULL)
+        break;
+    }
+
+  dbus_address_entries_free (entries);
+  return transport;
+}
+
+/**
+ * Creates a new transport for the "autostart" method.
+ * This creates a client-side of a transport.
+ *
+ * @param error address where an error can be returned.
+ * @returns a new transport, or #NULL on failure.
+ */
+static DBusTransport*
+_dbus_transport_new_for_autolaunch (DBusError      *error)
+{
+  DBusString address;
+  DBusTransport *result = NULL;
+
+  _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+
+  if (!_dbus_string_init (&address))
+    {
+      dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
+      return NULL;
+    }
+
+  if (!_dbus_get_autolaunch_address (&address, error))
+    {
+      _DBUS_ASSERT_ERROR_IS_SET (error);
+      goto out;
+    }
+
+  result = check_address (_dbus_string_get_const_data (&address), error);
+  if (result == NULL)
+    _DBUS_ASSERT_ERROR_IS_SET (error);
+  else
+    _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+
+ out:
+  _dbus_string_free (&address);
+  return result;
+}
+
+static DBusTransportOpenResult
+_dbus_transport_open_autolaunch (DBusAddressEntry  *entry,
+                                 DBusTransport    **transport_p,
+                                 DBusError         *error)
+{
+  const char *method;
+  
+  method = dbus_address_entry_get_method (entry);
+  _dbus_assert (method != NULL);
+
+  if (strcmp (method, "autolaunch") == 0)
+    {
+      *transport_p = _dbus_transport_new_for_autolaunch (error);
+
+      if (*transport_p == NULL)
+        {
+          _DBUS_ASSERT_ERROR_IS_SET (error);
+          return DBUS_TRANSPORT_OPEN_DID_NOT_CONNECT;
+        }
+      else
+        {
+          _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+          return DBUS_TRANSPORT_OPEN_OK;
+        }      
+    }
+  else
+    {
+      _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+      return DBUS_TRANSPORT_OPEN_NOT_HANDLED;
+    }
+}
+
+static const struct {
+  DBusTransportOpenResult (* func) (DBusAddressEntry *entry,
+                                    DBusTransport   **transport_p,
+                                    DBusError        *error);
+} open_funcs[] = {
+  { _dbus_transport_open_socket },
+  { _dbus_transport_open_platform_specific },
+  { _dbus_transport_open_autolaunch }
+#ifdef DBUS_BUILD_TESTS
+  , { _dbus_transport_open_debug_pipe }
+#endif
+};
+
 /**
  * Try to open a new transport for the given address entry.  (This
  * opens a client-side-of-the-connection transport.)
@@ -213,19 +331,14 @@ _dbus_transport_open (DBusAddressEntry *entry,
                       DBusError        *error)
 {
   DBusTransport *transport;
-  const char *address_problem_type;
-  const char *address_problem_field;
-  const char *address_problem_other;
-  const char *method;
   const char *expected_guid_orig;
   char *expected_guid;
-
+  int i;
+  DBusError tmp_error;
+  
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
   
   transport = NULL;
-  address_problem_type = NULL;
-  address_problem_field = NULL;
-  address_problem_other = NULL;
   expected_guid_orig = dbus_address_entry_get_value (entry, "guid");
   expected_guid = _dbus_strdup (expected_guid_orig);
 
@@ -234,115 +347,56 @@ _dbus_transport_open (DBusAddressEntry *entry,
       _DBUS_SET_OOM (error);
       return NULL;
     }
+
+  dbus_error_init (&tmp_error);
+  for (i = 0; i < (int) _DBUS_N_ELEMENTS (open_funcs); ++i)
+    {
+      DBusTransportOpenResult result;
+
+      _DBUS_ASSERT_ERROR_IS_CLEAR (&tmp_error);
+      result = (* open_funcs[i].func) (entry, &transport, &tmp_error);
+
+      switch (result)
+        {
+        case DBUS_TRANSPORT_OPEN_OK:
+          _DBUS_ASSERT_ERROR_IS_CLEAR (&tmp_error);
+          goto out;
+          break;
+        case DBUS_TRANSPORT_OPEN_NOT_HANDLED:
+          _DBUS_ASSERT_ERROR_IS_CLEAR (&tmp_error);
+          /* keep going through the loop of open funcs */
+          break;
+        case DBUS_TRANSPORT_OPEN_BAD_ADDRESS:
+          _DBUS_ASSERT_ERROR_IS_SET (&tmp_error);
+          goto out;
+          break;
+        case DBUS_TRANSPORT_OPEN_DID_NOT_CONNECT:
+          _DBUS_ASSERT_ERROR_IS_SET (&tmp_error);
+          goto out;
+          break;
+        }
+    }
+
+ out:
   
-  method = dbus_address_entry_get_method (entry);
-  _dbus_assert (method != NULL);
-      
-  if (strcmp (method, "unix") == 0)
-    {
-      const char *path = dbus_address_entry_get_value (entry, "path");
-      const char *tmpdir = dbus_address_entry_get_value (entry, "tmpdir");
-      const char *abstract = dbus_address_entry_get_value (entry, "abstract");
-          
-      if (tmpdir != NULL)
-        {
-          address_problem_other = "cannot use the \"tmpdir\" option for an address to connect to, only in an address to listen on";
-          goto bad_address;
-        }
-          
-      if (path == NULL && abstract == NULL)
-        {
-          address_problem_type = "unix";
-          address_problem_field = "path or abstract";  
-          goto bad_address;
-        }
-
-      if (path != NULL && abstract != NULL)
-        {
-          address_problem_other = "can't specify both \"path\" and \"abstract\" options in an address";
-          goto bad_address;
-        }
-
-      if (path)
-        transport = _dbus_transport_new_for_domain_socket (path, FALSE,
-                                                           error);
-      else
-        transport = _dbus_transport_new_for_domain_socket (abstract, TRUE,
-                                                           error);
-    }
-  else if (strcmp (method, "tcp") == 0)
-    {
-      const char *host = dbus_address_entry_get_value (entry, "host");
-      const char *port = dbus_address_entry_get_value (entry, "port");
-      DBusString  str;
-      long lport;
-      dbus_bool_t sresult;
-          
-      if (port == NULL)
-        {
-          address_problem_type = "tcp";
-          address_problem_field = "port";
-          goto bad_address;
-        }
-
-      _dbus_string_init_const (&str, port);
-      sresult = _dbus_string_parse_int (&str, 0, &lport, NULL);
-      _dbus_string_free (&str);
-          
-      if (sresult == FALSE || lport <= 0 || lport > 65535)
-        {
-          address_problem_other = "Port is not an integer between 0 and 65535";
-          goto bad_address;
-        }
-          
-      transport = _dbus_transport_new_for_tcp_socket (host, lport, error);
-    }
-#ifdef DBUS_BUILD_TESTS
-  else if (strcmp (method, "debug-pipe") == 0)
-    {
-      const char *name = dbus_address_entry_get_value (entry, "name");
-
-      if (name == NULL)
-        {
-          address_problem_type = "debug-pipe";
-          address_problem_field = "name";
-          goto bad_address;
-        }
-          
-      transport = _dbus_transport_debug_pipe_new (name, error);
-    }
-#endif
-  else
-    {
-      address_problem_other = "Unknown address type (examples of valid types are \"unix\" and \"tcp\")";
-      goto bad_address;
-    }
-
   if (transport == NULL)
     {
-      _DBUS_ASSERT_ERROR_IS_SET (error);
+      if (!dbus_error_is_set (&tmp_error))
+        _dbus_set_bad_address (&tmp_error,
+                               NULL, NULL,
+                               "Unknown address type (examples of valid types are \"tcp\" and on UNIX \"unix\")");
+      
+      _DBUS_ASSERT_ERROR_IS_SET (&tmp_error);
+      dbus_move_error(&tmp_error, error);
       dbus_free (expected_guid);
     }
   else
     {
+      _DBUS_ASSERT_ERROR_IS_CLEAR (&tmp_error);
       transport->expected_guid = expected_guid;
     }
-  
-  return transport;
-  
- bad_address:
-  dbus_free (expected_guid);
-  
-  if (address_problem_type != NULL)
-    dbus_set_error (error, DBUS_ERROR_BAD_ADDRESS,
-                    "Address of type %s was missing argument %s",
-                    address_problem_type, address_problem_field);
-  else
-    dbus_set_error (error, DBUS_ERROR_BAD_ADDRESS,
-                    "Could not parse address: %s",
-                    address_problem_other);
 
-  return NULL;
+  return transport;
 }
 
 /**
@@ -436,7 +490,16 @@ _dbus_transport_get_is_connected (DBusTransport *transport)
  */
 dbus_bool_t
 _dbus_transport_get_is_authenticated (DBusTransport *transport)
-{  
+{
+  /* We don't want to run unix_user_function on Windows, but it
+   * can exist, which allows application code to just unconditionally
+   * set it and have it only be invoked when appropriate.
+   */
+  dbus_bool_t on_windows = FALSE;
+#ifdef DBUS_WIN
+  on_windows = TRUE;
+#endif
+  
   if (transport->authenticated)
     return TRUE;
   else
@@ -508,7 +571,7 @@ _dbus_transport_get_is_authenticated (DBusTransport *transport)
 
           _dbus_auth_get_identity (transport->auth, &auth_identity);
 
-          if (transport->unix_user_function != NULL)
+          if (transport->unix_user_function != NULL && !on_windows)
             {
               dbus_bool_t allow;
               DBusConnection *connection;
@@ -523,11 +586,11 @@ _dbus_transport_get_is_authenticated (DBusTransport *transport)
 
               _dbus_verbose ("unlock %s\n", _DBUS_FUNCTION_NAME);
               _dbus_connection_unlock (connection);
-              
+
               allow = (* unix_user_function) (connection,
                                               auth_identity.uid,
                                               unix_user_data);
-
+              
               _dbus_verbose ("lock %s post unix user function\n", _DBUS_FUNCTION_NAME);
               _dbus_connection_lock (connection);
 
@@ -655,19 +718,19 @@ _dbus_transport_set_connection (DBusTransport  *transport,
 }
 
 /**
- * Get the UNIX file descriptor, if any.
+ * Get the socket file descriptor, if any.
  *
  * @param transport the transport
  * @param fd_p pointer to fill in with the descriptor
  * @returns #TRUE if a descriptor was available
  */
 dbus_bool_t
-_dbus_transport_get_unix_fd (DBusTransport *transport,
-                             int           *fd_p)
+_dbus_transport_get_socket_fd (DBusTransport *transport,
+                               int           *fd_p)
 {
   dbus_bool_t retval;
   
-  if (transport->vtable->get_unix_fd == NULL)
+  if (transport->vtable->get_socket_fd == NULL)
     return FALSE;
 
   if (transport->disconnected)
@@ -675,8 +738,8 @@ _dbus_transport_get_unix_fd (DBusTransport *transport,
 
   _dbus_transport_ref (transport);
 
-  retval = (* transport->vtable->get_unix_fd) (transport,
-                                               fd_p);
+  retval = (* transport->vtable->get_socket_fd) (transport,
+                                                 fd_p);
   
   _dbus_transport_unref (transport);
 
